const { v4: uuidv4 } = require('uuid');
const heroes = require('./heroes');
const ItemGenerator = require('../game_items/ItemGenerator.js');
const Balance = require('../../constants/Balance');
const { GAME_MODES, GAME_SUBMODES } = require('../../constants/Game');

const {
  GAME_SUBMODE_pvp1v1,
  GAME_SUBMODE_pvp2v2,
  GAME_SUBMODE_pvp3v3,
  GAME_SUBMODE_pvp5v5,
  GAME_SUBMODE_pvp7v7,
  GAME_SUBMODE_pvp11v11,
  GAME_SUBMODE_caverns1man,
  GAME_SUBMODE_caverns2man,
  GAME_SUBMODE_caverns3man,
  GAME_SUBMODE_caverns5man,
  GAME_SUBMODE_caverns7man,
  GAME_SUBMODE_caverns11man,
} = GAME_SUBMODES;

const INITIAL_INN_SUBMODE_STATE = {
  challenges_left: 10,
  current_tier: 1,
};

const INITIAL_CAVERNS_DATA = {
  deepest_level: 1,
  last_caverns_travel_at: 0,
  last_passive_yield_claim: 0,
  num_yield_slots: 1,
};

const NewPlayer = () => {
  // init hero roster
  const hero_roster = {};
  for (const hero_handle of Object.keys(heroes)) {
    hero_roster[hero_handle] = {
      handle: hero_handle,
      level: 0,
      xp: 0,
    };

    if (process.env.NODE_ENV === 'qa') {
      hero_roster[hero_handle].level =  50;
      hero_roster[hero_handle].xp = Balance.getXPForLevel(50);
    }
  }

  const caverns_data = {
    [GAME_SUBMODE_caverns1man]: INITIAL_CAVERNS_DATA,
    [GAME_SUBMODE_caverns2man]: INITIAL_CAVERNS_DATA,
    [GAME_SUBMODE_caverns3man]: INITIAL_CAVERNS_DATA,
    [GAME_SUBMODE_caverns5man]: INITIAL_CAVERNS_DATA,
    [GAME_SUBMODE_caverns7man]: INITIAL_CAVERNS_DATA,
    [GAME_SUBMODE_caverns11man]: INITIAL_CAVERNS_DATA,
  };

  const hero_builds = {};

  // init empty loadouts
  const loadouts = {
    [GAME_MODES.GAME_MODE_caverns]: {
      [GAME_SUBMODE_caverns1man]: [],
      [GAME_SUBMODE_caverns2man]: [],
      [GAME_SUBMODE_caverns3man]: [],
      [GAME_SUBMODE_caverns5man]: [],
      [GAME_SUBMODE_caverns7man]: [],
      [GAME_SUBMODE_caverns11man]: [],
    },
    [GAME_MODES.GAME_MODE_pvpAuto]: {
      [GAME_SUBMODE_pvp1v1]: [],
      [GAME_SUBMODE_pvp2v2]: [],
      [GAME_SUBMODE_pvp3v3]: [],
      [GAME_SUBMODE_pvp5v5]: [],
      [GAME_SUBMODE_pvp7v7]: [],
      [GAME_SUBMODE_pvp11v11]: [],
    },
    [GAME_MODES.GAME_MODE_pvpLive]: {
      [GAME_SUBMODE_pvp1v1]: [],
      [GAME_SUBMODE_pvp2v2]: [],
      [GAME_SUBMODE_pvp3v3]: [],
      [GAME_SUBMODE_pvp5v5]: [],
      [GAME_SUBMODE_pvp7v7]: [],
      [GAME_SUBMODE_pvp11v11]: [],
    },
  };

  // init player inventory
  const inventory = {
    bags: [{
      uid: uuidv4(),
      slot: 'item_container',
      type: 'bag',
      quality: 0.5,
      size: 20,
      items: [null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null],
    }],
  };
  // starter items
  for (let i = 0; i < Balance.NUM_STARTER_ITEMS; ++i) {
    inventory.bags[0].items[i] = ItemGenerator.getRandomItem(null, 0, 0.1, true);
  }

  // init altar data
  const altarState = {
    prayerSlots: [{ active: false }],
    alignment: Math.random(), // 0 = chaos, 0.5 = balance, 1.0 = order
    altarLevel: 1,
    altarXP: 0,
    chaosAffinity: 0,
    balanceAffinity: 0,
    orderAffinity: 0,
  };

  // init onboarding/tutorial state
  const tutorialProgress = [];

  // init pvpAuto submode states
  const player_inn_state = {
    [GAME_SUBMODE_pvp1v1]: INITIAL_INN_SUBMODE_STATE,
    [GAME_SUBMODE_pvp2v2]: INITIAL_INN_SUBMODE_STATE,
    [GAME_SUBMODE_pvp3v3]: INITIAL_INN_SUBMODE_STATE,
    [GAME_SUBMODE_pvp5v5]: INITIAL_INN_SUBMODE_STATE,
    [GAME_SUBMODE_pvp7v7]: INITIAL_INN_SUBMODE_STATE,
  };

  return {
    altarState,
    caverns_data,
    hero_builds,
    hero_roster,
    inventory,
    loadouts,
    player_inn_state,
    tutorialProgress,
  };
};
module.exports = NewPlayer;
